"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactQueryVisitor = exports.validate = exports.plugin = void 0;
const path_1 = require("path");
const graphql_1 = require("graphql");
const plugin_helpers_1 = require("@graphql-codegen/plugin-helpers");
const visitor_1 = require("./visitor");
Object.defineProperty(exports, "ReactQueryVisitor", { enumerable: true, get: function () { return visitor_1.ReactQueryVisitor; } });
const plugin = (schema, documents, config) => {
    const allAst = (0, graphql_1.concatAST)(documents.map(v => v.document));
    const allFragments = [
        ...allAst.definitions.filter(d => d.kind === graphql_1.Kind.FRAGMENT_DEFINITION).map(fragmentDef => {
            return {
                node: fragmentDef,
                name: fragmentDef.name.value,
                onType: fragmentDef.typeCondition.name.value,
                isExternal: false,
            };
        }),
        ...(config.externalFragments || []),
    ];
    const visitor = new visitor_1.ReactQueryVisitor(schema, allFragments, config, documents);
    // @ts-ignore strictFunctionTypes
    const visitorResult = (0, plugin_helpers_1.oldVisit)(allAst, { leave: visitor });
    // 產生 Keys
    const operationNames = documents.reduce((curr, row) => {
        row.document.definitions.map(x => {
            if ('name' in x && 'kind' in x && 'operation' in x) {
                if (x.operation === 'query' && x.name.kind === 'Name') {
                    curr.queryKey.push(`${x.name.value} = '${x.name.value}'`);
                }
                if (x.operation === 'mutation' && x.name.kind === 'Name') {
                    curr.mutationKey.push(`${x.name.value} = '${x.name.value}'`);
                }
            }
        });
        return curr;
    }, { queryKey: [], mutationKey: [] });
    if (visitor.hasOperations) {
        return {
            prepend: [...visitor.getImports(), visitor.getFetcherImplementation()],
            content: [
                `export enum EQueryKey{\n${operationNames.queryKey.join(',\n')}}`,
                `export enum EMutationKey{\n${operationNames.mutationKey.join(',\n')}}`,
                visitor.fragments,
                ...visitorResult.definitions.filter(t => typeof t === 'string'),
            ].join('\n'),
        };
    }
    return {
        prepend: [...visitor.getImports()],
        content: [
            visitor.fragments,
            ...visitorResult.definitions.filter(t => typeof t === 'string'),
        ].join('\n'),
    };
};
exports.plugin = plugin;
const validate = (schema, documents, config, outputFile) => __awaiter(void 0, void 0, void 0, function* () {
    if ((0, path_1.extname)(outputFile) !== '.ts' && (0, path_1.extname)(outputFile) !== '.tsx') {
        throw new Error('Plugin "typescript-react-query" requires extension to be ".ts" or ".tsx"!');
    }
});
exports.validate = validate;
//# sourceMappingURL=index.js.map